"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const class_transformer_1 = require("class-transformer");
const query_1 = require("../query");
/**
 * Base class for all query services that use a `typeorm` Repository.
 *
 * @example
 *
 * ```ts
 * @Injectable()
 * export class TodoItemService extends TypeOrmQueryService<TodoItemEntity> {
 *   constructor(
 *      @InjectRepository(TodoItemEntity) repo: Repository<TodoItemEntity>,
 *   ) {
 *     super(repo);
 *   }
 * }
 * ```
 */
class TypeOrmQueryService {
    /**
     * Creates a new QueryService for the passed in repository.
     * @param repo - the `typeorm` Repository.
     * @param filterQueryBuilder - **Should not need to provide through user code**. But allows you to specify the
     * FilterQueryBuilder, can be useful for testing.
     */
    constructor(repo, filterQueryBuilder = new query_1.FilterQueryBuilder(repo)) {
        this.repo = repo;
        this.filterQueryBuilder = filterQueryBuilder;
    }
    get entityType() {
        return this.repo.target;
    }
    /**
     * Query for multiple entities, using a Query from `@nestjs-query/core`.
     *
     * @example
     * ```ts
     * const todoItems = await this.service.query({
     *   filter: { title: { eq: 'Foo' } },
     *   paging: { limit: 10 },
     *   sorting: [{ field: "create", direction: SortDirection.DESC }],
     * });
     * ```
     * @param query - The Query used to filter, page, and sort rows.
     */
    query(query) {
        return this.filterQueryBuilder.select(query).getMany();
    }
    /**
     * Find an entity by it's `id`.
     *
     * @example
     * ```ts
     * const todoItem = await this.service.findById(1);
     * ```
     * @param id - The id of the record to find.
     */
    findById(id) {
        return this.repo.findOne(id);
    }
    /**
     * Gets an entity by it's `id`. If the entity is not found a rejected promise is returned.
     *
     * @example
     * ```ts
     * try {
     *   const todoItem = await this.service.getById(1);
     * } catch(e) {
     *   console.error('Unable to find entity with id = 1');
     * }
     * ```
     * @param id - The id of the record to find.
     */
    getById(id) {
        return this.repo.findOneOrFail(id);
    }
    /**
     * Creates a single entity.
     *
     * @example
     * ```ts
     * const todoItem = await this.service.createOne({title: 'Todo Item', completed: false });
     * ```
     * @param record - The entity to create.
     */
    createOne(record) {
        return this.repo.save(this.ensureIsEntity(record));
    }
    /**
     * Create multiple entities.
     *
     * @example
     * ```ts
     * const todoItem = await this.service.createMany([
     *   {title: 'Todo Item 1', completed: false },
     *   {title: 'Todo Item 2', completed: true },
     * ]);
     * ```
     * @param records - The entities to create.
     */
    createMany(records) {
        return this.repo.save(records.map(item => this.ensureIsEntity(item)));
    }
    /**
     * Update an entity.
     *
     * @example
     * ```ts
     * const updatedEntity = await this.service.updateOne(1, { completed: true });
     * ```
     * @param id - The `id` of the record.
     * @param update - A `Partial` of the entity with fields to update.
     */
    async updateOne(id, update) {
        const entity = await this.getById(id);
        return this.repo.save(this.repo.merge(entity, update));
    }
    /**
     * Update multiple entities with a `@nestjs-query/core` Filter.
     *
     * @example
     * ```ts
     * const { updatedCount } = await this.service.updateMany(
     *   { completed: true }, // the update to apply
     *   { title: { eq: 'Foo Title' } } // Filter to find records to update
     * );
     * ```
     * @param update - A `Partial` of entity with the fields to update
     * @param filter - A Filter used to find the records to update
     */
    async updateMany(update, filter) {
        const updateResult = await this.filterQueryBuilder
            .update({ filter })
            .set({ ...update })
            .execute();
        return { updatedCount: updateResult.affected || 0 };
    }
    /**
     * Delete an entity by `id`.
     *
     * @example
     *
     * ```ts
     * const deletedTodo = await this.service.deleteOne(1);
     * ```
     *
     * @param id - The `id` of the entity to delete.
     */
    async deleteOne(id) {
        const entity = await this.getById(id);
        return this.repo.remove(entity);
    }
    /**
     * Delete multiple records with a `@nestjs-query/core` `Filter`.
     *
     * @example
     *
     * ```ts
     * const { deletedCount } = this.service.deleteMany({
     *   created: { lte: new Date('2020-1-1') }
     * });
     * ```
     *
     * @param filter - A `Filter` to find records to delete.
     */
    async deleteMany(filter) {
        const deleteResult = await this.filterQueryBuilder.delete({ filter }).execute();
        return { deletedCount: deleteResult.affected || 0 };
    }
    /**
     * Covert an object to the `Entity` type if it is not an instance already.
     *
     * @param obj - The object to covert to an `Entity`
     */
    ensureIsEntity(obj) {
        if (obj instanceof this.entityType) {
            return obj;
        }
        return class_transformer_1.plainToClass(this.entityType, obj);
    }
}
exports.TypeOrmQueryService = TypeOrmQueryService;
//# sourceMappingURL=typeorm-query.service.js.map